%% Line and point-sampled intercept length calculator, Version 3a MTEX, Date 14.7.2021

%% Copyright and license
%{
Copyright (C) <2021>  <Pauli Lehto>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
%}

%% Requirements
% Tested to work with MTEX version 5.6 using Matlab R2021a.
% Note that earlier versions of MTEX require a bugfix for the MTEX file 'intersect.m'.
% See https://github.com/mtex-toolbox/mtex/issues/415 for issue description,
% and https://github.com/mtex-toolbox/mtex/tree/develop/EBSDAnalysis/%40grainBoundary for the fixed 'intersect.m' file

% The code requires the 'Statistics Toolbox' to run.

%% Version history and notes
% v3a MTEX: First public version.

% Note: Parallel for-loops are used to increase computational speed (to remove, replace 'parfor' with 'for' on lines: 221, 304, 388, 466, 557, 654, 745)
% Note: The code is developed with rectangular grid EBSD data (EBSDsquare). Initial attempts to account for hexagonal data (EBSDhex) have been added, but requires future work and checking.
% Note: Use the editor right-click sub-menu 'Code Folding->Fold All' to make the code look more tidy.

% The code is provided as is and can be freely modified.
% The version here represents the functionality used in Refs. [1] and [2]
% implemented here to work directly with EBSD data instead of image-based analysis.
% The work is related to International Institute of Welding Commission C-IX, document nr. IX-L-1239-2021.

%% Usage instructions
% 1. Download and install MTEX 5.6, navigate to the path of MTEX, and execute command "install_mtex".
% 2. Import EBSD data as variable named 'ebsd'. Example data can be loaded using e.g. the command 'mtexdata csl;'.
% 3. Check the parameters set at sections 'Linear intercept parameters' and 'EBSD processing parameters'. Typical parameters are set as default.
% 4. Run the code (F5).
% 5. For statistical analysis use the distribution fitting toolbox 'dfittool', refer to the the 'List of output variables' below.

%% List of output variables:
%{
    average_grainsize: average grain size, d. Measured using line-sampled linear intercept length method, see Ref. [1]
    Dd_d: Relative grain size dispersion. Measured using line-sampled linear intercept length method, see Ref. [1]
    volume_weighted_average: volume-weighted average grain size, dv. Measured using point-sampled linear intercept length method, see Ref. [1]

    grainsize: Variable containing all measurements for the line-sampled method
    grainsize_probability_level: Grain size at probability levels 1-99.9%, measured using the line-sampled method

    grainsize_all_dirs_ps: Variable containing all measurements for the point-sampled method
    grainsize_mean_ps: Mean value of the four measurement directions for all datapoints in the EBSD map, measured using the point-sampled method
    gsmap_ : Grain size maps (in m) for the four measurement directions, the mean of four directions, and the minimum and maximum value of the four directions, measured using the point-sampled method

    mov_avg_ : Variables for moving averages of grain size in horizontal (x) and vertical (y) directions. Individual traces from gsmap_mean, gsmap_min and gsmap_max, measured using the point-sampled method.
%}

%% ------------------------Coded by------------------------------
% Pauli Lehto
% Staff Scientist, D.Sc. (Tech.)
% Aalto University, School of Engineering
% Department of Mechanical Engineering
% Visiting address: Otakaari 4, 02150 Espoo
% P.O. Box 14100
% e-mail: pauli.lehto@aalto.fi
%
% For futher information about the measurement procedure please refer to:
% [1] P. Lehto et al., Influence of grain size distribution on the Hall-Petch relationship of welded structural steel,
% Materials Science and Engineering: A, 2014; 592: 28-39, http://dx.doi.org/10.1016/j.msea.2013.10.094
% [2] P. Lehto et al. Characterisation of local grain size variation of welded structural steel.
% Welding in the World. 2016; 60: 673-678. http://dx.doi.org/10.1007/s40194-016-0318-8
% [3] P. Lehto, Aalto University Wiki - Grain size measurement using Matlab. https://wiki.aalto.fi/display/GSMUM
% --------------------------------------------------------------

%% Linear intercept parameters
gb_threshold=10*degree; %Misorientation threshold in degrees for grain boundaries
include_innerGB = true; %Include the non-continuous GB segments in the measurement

margin = 2; %Margin at edges of EBSD map in pixels, has to be =>1
averaging_length = 10; %Width of rectangular probe for moving averages, set as number of adjacent datapoints.

figure_show = true; %Overlay grain size on GB map and display moving averages
gb_width = 1; %Thickness of GB's in the plots

%% EBSD processing parameters
remove_wildspikes = false; %If orientation data has a lot of wild orientation spikes, they will create small artificial grains. Wild spikes (and grains smaller than 3 px) removed if set to true.
denoise_data = false; %If true, half-quadratic de-noising is carried out according to the parameters published in Lehto [Ultramicroscopy 2021]. Recommended with wild spike removal.
%See: Hielscher et al. 2019, https://doi.org/10.1107/S1600576719009075 and https://mtex-toolbox.github.io/BoundaryPlots.html (% remove very small %grains, % and recompute grains before de-noising)

%% MTEX plotting convention
setMTEXpref('xAxisDirection','east');
setMTEXpref('zAxisDirection','intoPlane');
setMTEXpref('figSize','normal');

%% Input data
% Import ebsd data as workspace variable 'ebsd' before running the code
% For testing, find example datasets with command 'mtexdata', for example:
% mtexdata csl;
% mtexdata titanium;

%% Wild spikes removal and data de-noising
for zz=1:1
    if remove_wildspikes==true
        tic
        fprintf('Removing wild orientation spikes. \n');
        [grains,ebsd.grainId,~] = calcGrains(ebsd('Indexed'),'angle',gb_threshold);
        ebsd(grains(grains.grainSize<=3)) = []; %<3 pixel grains removed
        a = toc;
        fprintf('Elapsed time %.0f seconds.\n', a);
    end
    
    if denoise_data==true
        tic
        % Apply half quadratic filtering if needed (MTEX V5.3 definitions)
        F = halfQuadraticFilter;
        % Main filtering parameter, higher values mean more smoothing
        F.alpha = 1;            % regularization parameter, typical value 1
        
        % Other filtering parameters
        F.l1DataFit = true;      % use l^1 norm for data fitting
        F.l1TV      = true;      % use l^1 norm for regularization
        F.iterMax   = 5000;     % maximum number of iterations
        F.tol   = 0.02*degree;   % stopping criterium for the gradient descent
        F.eps   = 1e-3;         % l^1 relaxation parameter
        F.threshold = 5*degree; % threshold for subgrain boundaries %(no filtering when point-to-point misorientation less than 2 degrees)
        
        fprintf('Smoothing ebsd data. \n');
        ebsd_smoothed = smooth(ebsd('indexed'),F,'fill'); %Filter and Fill all non-indexed pixels
        ebsd_grid=gridify(ebsd_smoothed);
        a = toc;
        fprintf('Elapsed time %.0f seconds.\n', a);
    end
    
    %Grid data
    try
        ebsd_grid=gridify(ebsd_smoothed);
    catch
        fprintf('No de-noising has been carried out, using raw data.\n');
        ebsd_grid=gridify(ebsd);
    end
end

%% Calculate grains
for zz=1:1
    tic
    fprintf(['Calculating grain structure, GBs: ' num2str(gb_threshold/degree) ' deg. ']);
    [grains_grid,ebsd_grid.grainId,ebsd_grid.mis2mean] = calcGrains(ebsd_grid('Indexed'),'angle',gb_threshold);
    % grains_grid=smooth(grains_grid,3); %Smoothing of grain boundaries if desired
    
    grain_ids=find(grains_grid.grainSize>1); %Find grains larger than 1 pixel for plotting
    a = toc;
    fprintf('Elapsed time %.0f seconds.\n', a);
end

%% Grain size measurement
for zz=1:1
    %% Initialize linear intercept variables
    for zx=1:1
        %Linear intercept parameters
        spacing_vert = 7; %Vertical intercept line spacing in pixels, for information about convergence refer to [1].
        spacing_hor = spacing_vert; %Horizontal intercept line spacing, by default equal to vertical (equal line length measured in both directions)
        spacing_45 = 10; %45 degree line spacing, should be vertical/0.7 (equal line length as horizontal and vertical). (Sin 45 degrees=0.707 defines the perpendicular distance between measurement lines)
        min_measurement = 2; %Minimum number of pixels required between adjacent intercept points, consistent with Ref. [1]
        
        [d1,d2]=size(ebsd_grid);
        x_grid=ebsd_grid.x;
        y_grid=ebsd_grid.y;
        stepsize_dx=ebsd_grid.dx;
        stepsize_dy=ebsd_grid.dy;
        gb=grains_grid.boundary; %Complete grains with continuous GBs
        gb_inner=grains_grid.innerBoundary; %Incomplete GBs
        
        grainsize_hor = [];
        grainsize_ver = [];
        grainsize_45_1 = [];
        grainsize_45_2 = [];
        intercept_x_hor={};
        intercept_y_hor={};
        intercept_x_ver={};
        intercept_y_ver={};
        intercept_x_45_1={};
        intercept_y_45_1={};
        intercept_x_45_2={};
        intercept_y_45_2={};
        
        % Write map x,y indices into arrays
        point_x = repmat(1:d2,d1,1);
        point_y = repmat(transpose(1:d1),1,d2);
        
        %Logical indexing to be for efficiency (Usually equivalent to ebsd.id, but may not always be the case)
        logical_ids=1:d1*d2;
        ebsd_ids=transpose(reshape(logical_ids,d1,d2))';
        % ebsd_ids=ebsd_grid.id; %Assumes that first index is top left corner, increasing downward (consistent with matlab index)
        
        %Initialize gs maps to the correct size
        gsmap_hor=zeros(d1,d2);
        gsmap_ver=zeros(d1,d2);
        gsmap_45_1=zeros(d1,d2);
        gsmap_45_2=zeros(d1,d2);
        
        %Grain size quantiles to use as color scale limits
        plot_quantile_ub = 0.99; %Upper bound quantile grain size for the color scale.
        plot_quantile_lb = 0.01; %Lower bound quantile grain size for the color scale.
    end
    
    %Format variables for each iteration
    fprintf('<strong>Measuring line and point-sampled intercept length simultaneously.</strong>\n');
    
    %% Horizontal linear intercept
    fprintf('Measuring horizontal grain size. ');
    tic;
    linesampled_locs=margin+1:spacing_vert:d1-margin;
    tempvar_writeindex={};
    tempvar_grainsizes={};
    tempvar_lineids={};
    parfor i=margin+1:d1-margin
        %Define start and endpoints for the intercept line
        line_ids=ebsd_ids(i, margin+1:d2-margin);
        point_x_line=x_grid(line_ids);
        point_y_line=y_grid(line_ids);
        xy1 = [point_x_line(1),point_y_line(1)];
        xy2 = [point_x_line(end),point_y_line(end)];
        
        %Measure intercepts for grain boundary
        [x_gb,y_gb,~] = intersect(gb,xy1,xy2);
        x_gb=x_gb(~isnan(x_gb));
        y_gb=y_gb(~isnan(y_gb));
        
        if include_innerGB == true
            %Measure intercepts for inner grain boundary
            [x_igb,y_igb,~] = intersect(gb_inner,xy1,xy2);
            x_igb=x_igb(~isnan(x_igb));
            y_igb=y_igb(~isnan(y_igb));
            
            if isempty(x_igb) == 0
                %Combine the measurements
                x_gb=horzcat(x_gb,x_igb);
                y_gb=horzcat(y_gb,y_igb);
                [x_gb,sort_index]=sort(x_gb);
                y_gb=y_gb(sort_index);
            else
                [x_gb,sort_index]=sort(x_gb);
                y_gb=y_gb(sort_index);
            end
        else
            [x_gb,sort_index]=sort(x_gb);
            y_gb=y_gb(sort_index);
        end
        
        %Find the closest datapoint based on x y coordinates
        x_gb=x_gb+stepsize_dx/2; %Aadd half of stepsize to shift measuremens onto actual datapoints (for plotting purposes).
        
        %If intersect touches image margin, remove
        [~,rem_x_index]=find(x_gb==point_x_line(1) | x_gb==point_x_line(end));
        x_gb(rem_x_index)=[];
        y_gb(rem_x_index)=[];
        [minValue, closestIndex] = min(abs(x_gb - point_x_line.'));
        
        %Calculate grain size
        grainsize_hor_temp=diff(x_gb);
        
        %Save values to temporary variables
        if isempty(closestIndex) == 0
            tempvar_writeindex{i}=closestIndex;
            tempvar_grainsizes{i}=grainsize_hor_temp;
            tempvar_lineids{i}=line_ids;
        end
        
        %Only write the grain size values that are consistent with 7:10
        %line-spacing
        [~,valfound]=find(linesampled_locs==i);
        if isempty(grainsize_hor_temp) == 0 && isempty(valfound) ==0
            grainsize_hor=horzcat(grainsize_hor,grainsize_hor_temp);
        end
    end
    
    %Write values to GS map
    for i=1:length(tempvar_writeindex)
        closestIndex=tempvar_writeindex{i};
        grainsize_temp=tempvar_grainsizes{i};
        line_ids=tempvar_lineids{i};
        if isempty(closestIndex) == 0
            for t=1:length(grainsize_temp)
                write_index=closestIndex(t):closestIndex(t+1); %Output index
                if length(write_index)>min_measurement %At least n pixels required between the measurements
                    write_index(end)=[]; %Remove last index, so last measurement ends on the grain interior side
                    gsmap_hor(line_ids(write_index))=grainsize_temp(t);
                end
            end
        end
    end
    clear tempvar_writeindex tempvar_grainsizes tempvar_lineids;
    a = toc;
    fprintf('Elapsed time %.0f seconds.\n', a);
    
    %% Vertical linear intercept
    fprintf('Measuring vertical grain size. ');
    tic
    linesampled_locs=margin+1:spacing_hor:d1-margin;
    tempvar_writeindex={};
    tempvar_grainsizes={};
    tempvar_lineids={};
    
    parfor i=margin+1:d2-margin
        %Define start and endpoints for the intercept line
        line_ids=ebsd_ids(margin+1:d1-margin,i);
        point_x_line=x_grid(line_ids);
        point_y_line=y_grid(line_ids);
        xy1 = [point_x_line(1),point_y_line(1)];
        xy2 = [point_x_line(1),point_y_line(end)];
        
        %Measure intercepts for grain boundary
        [x_gb,y_gb,~] = intersect(gb,xy1,xy2);
        x_gb=x_gb(~isnan(x_gb));
        y_gb=y_gb(~isnan(y_gb));
        
        if include_innerGB == true
            %Measure intercepts for inner grain boundary
            [x_igb,y_igb,~] = intersect(gb_inner,xy1,xy2);
            x_igb=x_igb(~isnan(x_igb));
            y_igb=y_igb(~isnan(y_igb));
            
            if isempty(y_igb) == 0
                %Combine the measurements
                x_gb=horzcat(x_gb,x_igb);
                y_gb=horzcat(y_gb,y_igb);
                [y_gb,sort_index]=sort(y_gb);
                x_gb=x_gb(sort_index);
            else
                [y_gb,sort_index]=sort(y_gb);
                x_gb=x_gb(sort_index);
            end
        else
            [y_gb,sort_index]=sort(y_gb);
            x_gb=x_gb(sort_index);
        end
        
        %Find the closest datapoint based on x y coordinates
        y_gb=y_gb+stepsize_dy/2; %Aadd half of stepsize to shift measuremens onto actual datapoints (for plotting purposes).
        %If intersect touches image margin, remove
        [~,rem_y_index]=find(y_gb==point_y_line(1) | y_gb==point_y_line(end));
        x_gb(rem_y_index)=[];
        y_gb(rem_y_index)=[];
        [minValue, closestIndex] = min(abs(y_gb - transpose(point_y_line).'));
        
        %Calculate grain size
        grainsize_ver_temp=diff(y_gb);
        
        %Save values to temporary variables
        if isempty(closestIndex) == 0
            tempvar_writeindex{i}=closestIndex;
            tempvar_grainsizes{i}=grainsize_ver_temp;
            tempvar_lineids{i}=line_ids;
        end
        
        %Only write the grain size values that are consistent with 7:10
        %line-spacing
        [~,valfound]=find(linesampled_locs==i);
        if isempty(grainsize_ver_temp) == 0 && isempty(valfound) ==0
            grainsize_ver=horzcat(grainsize_ver,grainsize_ver_temp);
        end
    end
    
    %Write values to GS map
    for i=1:length(tempvar_writeindex)
        closestIndex=tempvar_writeindex{i};
        grainsize_temp=tempvar_grainsizes{i};
        line_ids=tempvar_lineids{i};
        if isempty(closestIndex) == 0
            for t=1:length(grainsize_temp)
                write_index=closestIndex(t):closestIndex(t+1); %Output index
                if length(write_index)>min_measurement %At least n pixels required between the measurements
                    write_index(end)=[]; %Remove last index, so last measurement ends on the grain interior side
                    gsmap_ver(line_ids(write_index))=grainsize_temp(t);
                end
            end
        end
    end
    clear tempvar_writeindex tempvar_grainsizes tempvar_lineids;
    a = toc;
    fprintf('Elapsed time %.0f seconds.\n', a);
    
    %% Hexagonal data: additional vertical linear intercept to account for the interlocking grid (x value interval = 0.5*ebsd_grid.dx)
    %Only for visualisation purposes. Not considered for the line-sampled intercept length
    if isa(ebsd_grid,'EBSDhex') == 1
        fprintf('Hexagonal data: additional vertical measurement. ');
        tic
        tempvar_writeindex={};
        tempvar_grainsizes={};
        tempvar_lineids={};
        
        parfor i=margin+1:d2-margin
            %Define start and endpoints for the intercept line
            line_ids=ebsd_ids(margin+1:d1-margin,i);
            point_x_line=x_grid(line_ids);
            point_y_line=y_grid(line_ids);
            xy1 = [point_x_line(2),point_y_line(1)];
            xy2 = [point_x_line(2),point_y_line(end)];
            
            %Measure intercepts for grain boundary
            [x_gb,y_gb,~] = intersect(gb,xy1,xy2);
            x_gb=x_gb(~isnan(x_gb));
            y_gb=y_gb(~isnan(y_gb));
            
            if include_innerGB == true
                %Measure intercepts for inner grain boundary
                [x_igb,y_igb,~] = intersect(gb_inner,xy1,xy2);
                x_igb=x_igb(~isnan(x_igb));
                y_igb=y_igb(~isnan(y_igb));
                
                if isempty(y_igb) == 0
                    %Combine the measurements
                    x_gb=horzcat(x_gb,x_igb);
                    y_gb=horzcat(y_gb,y_igb);
                    [y_gb,sort_index]=sort(y_gb);
                    x_gb=x_gb(sort_index);
                else
                    [y_gb,sort_index]=sort(y_gb);
                    x_gb=x_gb(sort_index);
                end
            else
                [y_gb,sort_index]=sort(y_gb);
                x_gb=x_gb(sort_index);
            end
            
            %Find the closest datapoint based on x y coordinates
            y_gb=y_gb+stepsize_dy/2; %Aadd half of stepsize to shift measuremens onto actual datapoints (for plotting purposes).
            %If intersect touches image margin, remove
            [~,rem_y_index]=find(y_gb==point_y_line(1) | y_gb==point_y_line(end));
            x_gb(rem_y_index)=[];
            y_gb(rem_y_index)=[];
            [minValue, closestIndex] = min(abs(y_gb - transpose(point_y_line).'));
            
            %Calculate grain size
            grainsize_ver_temp=diff(y_gb);
            
            %Save values to temporary variables
            if isempty(closestIndex) == 0
                tempvar_writeindex{i}=closestIndex;
                tempvar_grainsizes{i}=grainsize_ver_temp;
                tempvar_lineids{i}=line_ids;
            end
        end
        
        %Write values to GS map
        for i=1:length(tempvar_writeindex)
            closestIndex=tempvar_writeindex{i};
            grainsize_temp=tempvar_grainsizes{i};
            line_ids=tempvar_lineids{i};
            if isempty(closestIndex) == 0
                for t=1:length(grainsize_temp)
                    write_index=closestIndex(t):closestIndex(t+1); %Output index
                    if length(write_index)>min_measurement %At least n pixels required between the measurements
                        write_index(end)=[]; %Remove last index, so last measurement ends on the grain interior side
                        gsmap_ver(line_ids(write_index))=grainsize_temp(t);
                    end
                end
            end
        end
        clear tempvar_writeindex tempvar_grainsizes tempvar_lineids;
        a = toc;
        fprintf('Elapsed time %.0f seconds.\n', a);
    end
    
    %% 45 deg linear intercept 1: left->right, top->bottom, i.e. \
    fprintf('Measuring 45 degree grain size #1. ');
    tic
    linesampled_locs=margin+1:spacing_45:d1-margin-1;
    tempvar_writeindex={};
    tempvar_grainsizes={};
    tempvar_lineids={};
    
    %Measurement below diagonal
    parfor i=margin+1:d1-margin-1 %Start in top left corner, increase y
        %Crop xy data and logical ID's such that the corner of the data matches
        %with the starting point (can use diagonal matrix)
        line_ids=diag(ebsd_ids(i:d1-margin,margin:d2-margin));
        point_x_line=diag(x_grid(i:d1-margin,margin:d2-margin));
        point_y_line=diag(y_grid(i:d1-margin,margin:d2-margin));
        xy1 = [point_x_line(1),point_y_line(1)];
        xy2 = [point_x_line(end),point_y_line(end)];
        
        %Measure intercepts for grain boundary
        [x_gb,y_gb,~] = intersect(gb,xy1,xy2);
        x_gb=x_gb(~isnan(x_gb));
        y_gb=y_gb(~isnan(y_gb));
        
        if include_innerGB == true
            %Measure intercepts for inner grain boundary
            [x_igb,y_igb,~] = intersect(gb_inner,xy1,xy2);
            x_igb=x_igb(~isnan(x_igb));
            y_igb=y_igb(~isnan(y_igb));
            
            if isempty(x_igb) == 0
                %Combine the measurements
                x_gb=horzcat(x_gb,x_igb);
                y_gb=horzcat(y_gb,y_igb);
                [x_gb,sort_index]=sort(x_gb);
                y_gb=y_gb(sort_index);
            else
                [x_gb,sort_index]=sort(x_gb);
                y_gb=y_gb(sort_index);
            end
        else
            [x_gb,sort_index]=sort(x_gb);
            y_gb=y_gb(sort_index);
        end
        
        %Find unique values (some problem in MTEX with duplicates)
        [x_gb_unique,ia,ic] = unique(x_gb);
        y_gb_unique=y_gb(ia);
        
        %If intersect touches image margin, remove
        [~,rem_x_index]=find(x_gb_unique==point_x_line(1) | x_gb_unique==point_x_line(end));
        x_gb_unique(rem_x_index)=[];
        y_gb_unique(rem_x_index)=[];
        [~,rem_y_index]=find(y_gb_unique==point_y_line(1) | y_gb_unique==point_y_line(end));
        x_gb_unique(rem_y_index)=[];
        y_gb_unique(rem_y_index)=[];
        
        %Additional removing (somehow unique doesnt remove everything)
        [~,id_rem]=find(diff(x_gb_unique)<0.1*stepsize_dx); %Delete points very close together
        x_gb_unique(id_rem)=[];
        y_gb_unique(id_rem)=[];
        
        %Find the closest datapoint based on x y coordinates
        x_gb_unique=x_gb_unique+stepsize_dx/2; %Aadd half of stepsize to shift measuremens onto actual datapoints (for plotting purposes).
        y_gb_unique=y_gb_unique+stepsize_dy/2; %Aadd half of stepsize to shift measuremens onto actual datapoints (for plotting purposes).
        [minValue, closestIndex] = min(abs(x_gb_unique - transpose(point_x_line).')); %Find closest indices, only need to do one direction
        
        %Calculate grain size
        hor_dist_45=diff(x_gb_unique);
        ver_dist_45=diff(y_gb_unique);
        grainsize_45_1_temp = sqrt((hor_dist_45.^2)+(ver_dist_45.^2));
        
        %Save values to temporary variables
        if isempty(closestIndex) == 0
            tempvar_writeindex{i}=closestIndex;
            tempvar_grainsizes{i}=grainsize_45_1_temp;
            tempvar_lineids{i}=line_ids;
        end
        
        %Only write the grain size values that are consistent with 7:10 line-spacing
        [~,valfound]=find(linesampled_locs==i);
        if isempty(grainsize_45_1_temp) == 0 && isempty(valfound) ==0
            grainsize_45_1=horzcat(grainsize_45_1,grainsize_45_1_temp);
        end
    end
    
    %Write values to GS map
    for i=1:length(tempvar_writeindex)
        closestIndex=tempvar_writeindex{i};
        grainsize_temp=tempvar_grainsizes{i};
        line_ids=tempvar_lineids{i};
        if isempty(closestIndex) == 0
            for t=1:length(grainsize_temp)
                write_index=closestIndex(t):closestIndex(t+1); %Output index
                if length(write_index)>min_measurement %At least n pixels required between the measurements
                    write_index(end)=[]; %Remove last index, so last measurement ends on the grain interior side
                    gsmap_45_1(line_ids(write_index))=grainsize_temp(t);
                end
            end
        end
    end
    clear tempvar_writeindex tempvar_grainsizes tempvar_lineids;
    
    linesampled_locs=margin:spacing_45:d2-margin-1;
    tempvar_writeindex={};
    tempvar_grainsizes={};
    tempvar_lineids={};
    
    %Measurement above diagonal
    parfor i=margin:d2-margin-1 %Start in top left corner, increase x
        %Crop xy data and logical ID's such that the corner of the data matches
        %with the starting point (can use diagonal matrix)
        line_ids=diag(ebsd_ids(margin:d1-margin,i:d2-margin));
        point_x_line=diag(x_grid(margin:d1-margin,i:d2-margin));
        point_y_line=diag(y_grid(margin:d1-margin,i:d2-margin));
        xy1 = [point_x_line(1),point_y_line(1)];
        xy2 = [point_x_line(end),point_y_line(end)];
        
        %Measure intercepts for grain boundary
        [x_gb,y_gb,~] = intersect(gb,xy1,xy2);
        x_gb=x_gb(~isnan(x_gb));
        y_gb=y_gb(~isnan(y_gb));
        
        if include_innerGB == true
            %Measure intercepts for inner grain boundary
            [x_igb,y_igb,~] = intersect(gb_inner,xy1,xy2);
            x_igb=x_igb(~isnan(x_igb));
            y_igb=y_igb(~isnan(y_igb));
            
            if isempty(x_igb) == 0
                %Combine the measurements
                x_gb=horzcat(x_gb,x_igb);
                y_gb=horzcat(y_gb,y_igb);
                [x_gb,sort_index]=sort(x_gb);
                y_gb=y_gb(sort_index);
            else
                [x_gb,sort_index]=sort(x_gb);
                y_gb=y_gb(sort_index);
            end
        else
            [x_gb,sort_index]=sort(x_gb);
            y_gb=y_gb(sort_index);
        end
        
        %Find unique values (some problem in MTEX with duplicates)
        [x_gb_unique,ia,ic] = unique(x_gb);
        y_gb_unique=y_gb(ia);
        
        %Additional removing (somehow unique doesnt remove everything)
        [~,id_rem]=find(diff(x_gb_unique)<0.1*stepsize_dx); %Delete points very close together
        x_gb_unique(id_rem)=[];
        y_gb_unique(id_rem)=[];
        
        %If intersect touches image margin, remove
        [~,rem_x_index]=find(x_gb_unique==point_x_line(1) | x_gb_unique==point_x_line(end));
        x_gb_unique(rem_x_index)=[];
        y_gb_unique(rem_x_index)=[];
        [~,rem_y_index]=find(y_gb_unique==point_y_line(1) | y_gb_unique==point_y_line(end));
        x_gb_unique(rem_y_index)=[];
        y_gb_unique(rem_y_index)=[];
        
        %Find the closest datapoint based on x y coordinates
        x_gb_unique=x_gb_unique+stepsize_dx/2; %Aadd half of stepsize to shift measuremens onto actual datapoints (for plotting purposes).
        y_gb_unique=y_gb_unique+stepsize_dy/2; %Aadd half of stepsize to shift measuremens onto actual datapoints (for plotting purposes).
        [minValue, closestIndex] = min(abs(x_gb_unique - transpose(point_x_line).')); %Find closest indices, only need to do one direction
        
        %Calculate grain size
        hor_dist_45=diff(x_gb_unique);
        ver_dist_45=diff(y_gb_unique);
        grainsize_45_1_temp = sqrt((hor_dist_45.^2)+(ver_dist_45.^2));
        
        %Save values to temporary variables
        if isempty(closestIndex) == 0
            tempvar_writeindex{i}=closestIndex;
            tempvar_grainsizes{i}=grainsize_45_1_temp;
            tempvar_lineids{i}=line_ids;
        end
        
        %Only write the grain size values that are consistent with 7:10 line-spacing
        [~,valfound]=find(linesampled_locs==i);
        if isempty(grainsize_45_1_temp) == 0 && isempty(valfound) ==0
            grainsize_45_1=horzcat(grainsize_45_1,grainsize_45_1_temp);
        end
    end
    
    %Write values to GS map
    for i=1:length(tempvar_writeindex)
        closestIndex=tempvar_writeindex{i};
        grainsize_temp=tempvar_grainsizes{i};
        line_ids=tempvar_lineids{i};
        if isempty(closestIndex) == 0
            for t=1:length(grainsize_temp)
                write_index=closestIndex(t):closestIndex(t+1); %Output index
                if length(write_index)>min_measurement %At least n pixels required between the measurements
                    write_index(end)=[]; %Remove last index, so last measurement ends on the grain interior side
                    gsmap_45_1(line_ids(write_index))=grainsize_temp(t);
                end
            end
        end
    end
    clear tempvar_writeindex tempvar_grainsizes tempvar_lineids;
    
    a = toc;
    fprintf('Elapsed time %.0f seconds.\n', a);
    
    %% 45 deg linear intercept 2: left->right, bottom->top, i.e. /
    fprintf('Measuring 45 degree grain size #2. ');
    tic
    
    linesampled_locs=margin+1:spacing_45:d1-margin;
    tempvar_writeindex={};
    tempvar_grainsizes={};
    tempvar_lineids={};
    %Measurement above diagonal
    parfor i=margin+1:d1-margin %Start in bottom left corner, decrease y
        %Crop xy data and logical ID's such that the corner of the data matches
        %with the starting point (can use diagonal matrix)
        %Flip matrices to get the correct diagonal
        line_ids=diag(flip(ebsd_ids(margin:i,margin:d2-margin)));
        point_x_line=diag(flip(x_grid(margin:i,margin:d2-margin)));
        point_y_line=diag(flip(y_grid(margin:i,margin:d2-margin)));
        xy1 = [point_x_line(1),point_y_line(1)];
        xy2 = [point_x_line(end),point_y_line(end)];
        
        %Measure intercepts for grain boundary
        [x_gb,y_gb,~] = intersect(gb,xy1,xy2);
        x_gb=x_gb(~isnan(x_gb));
        y_gb=y_gb(~isnan(y_gb));
        
        if include_innerGB == true
            %Measure intercepts for inner grain boundary
            [x_igb,y_igb,~] = intersect(gb_inner,xy1,xy2);
            x_igb=x_igb(~isnan(x_igb));
            y_igb=y_igb(~isnan(y_igb));
            
            if isempty(x_igb) == 0
                %Combine the measurements
                x_gb=horzcat(x_gb,x_igb);
                y_gb=horzcat(y_gb,y_igb);
                [x_gb,sort_index]=sort(x_gb);
                y_gb=y_gb(sort_index);
            else
                [x_gb,sort_index]=sort(x_gb);
                y_gb=y_gb(sort_index);
            end
        else
            [x_gb,sort_index]=sort(x_gb);
            y_gb=y_gb(sort_index);
        end
        
        %Find unique values (some problem in MTEX with duplicates)
        [x_gb_unique,ia,ic] = unique(x_gb);
        y_gb_unique=y_gb(ia);
        
        %Additional removing (somehow unique doesnt remove everything)
        [~,id_rem]=find(abs(diff(y_gb_unique))<0.1*stepsize_dy); %Delete points very close together
        x_gb_unique(id_rem)=[];
        y_gb_unique(id_rem)=[];
        
        %If intersect touches image margin, remove
        [~,rem_x_index]=find(x_gb_unique==point_x_line(1) | x_gb_unique==point_x_line(end));
        x_gb_unique(rem_x_index)=[];
        y_gb_unique(rem_x_index)=[];
        [~,rem_y_index]=find(y_gb_unique==point_y_line(1) | y_gb_unique==point_y_line(end));
        x_gb_unique(rem_y_index)=[];
        y_gb_unique(rem_y_index)=[];
        
        %Find the closest datapoint based on x y coordinates
        x_gb_unique=x_gb_unique+stepsize_dx/2; %Add half of stepsize to shift measuremens onto actual datapoints (for plotting purposes, matches direction of measurement).
        y_gb_unique=y_gb_unique-stepsize_dy/2; %Substract half of stepsize to shift measuremens onto actual datapoints (for plotting purposes, matches direction of measurement).
        [minValue, closestIndex] = min(abs(x_gb_unique - transpose(point_x_line).')); %Find closest indices, only need to do one direction
        
        %Calculate grain size
        hor_dist_45=diff(x_gb_unique);
        ver_dist_45=diff(y_gb_unique);
        grainsize_45_2_temp = sqrt((hor_dist_45.^2)+(ver_dist_45.^2));
        
        %Save values to temporary variables
        if isempty(closestIndex) == 0
            tempvar_writeindex{i}=closestIndex;
            tempvar_grainsizes{i}=grainsize_45_2_temp;
            tempvar_lineids{i}=line_ids;
        end
        
        %Only write the grain size values that are consistent with 7:10 line-spacing
        [~,valfound]=find(linesampled_locs==i);
        if isempty(grainsize_45_2_temp) == 0 && isempty(valfound) ==0
            grainsize_45_2=horzcat(grainsize_45_2,grainsize_45_2_temp);
        end
    end
    
    %Write values to GS map
    for i=1:length(tempvar_writeindex)
        closestIndex=tempvar_writeindex{i};
        grainsize_temp=tempvar_grainsizes{i};
        line_ids=tempvar_lineids{i};
        if isempty(closestIndex) == 0
            for t=1:length(grainsize_temp)
                write_index=closestIndex(t):closestIndex(t+1); %Output index
                if length(write_index)>min_measurement %At least n pixels required between the measurements
                    write_index(end)=[]; %Remove last index, so last measurement ends on the grain interior side
                    gsmap_45_2(line_ids(write_index))=grainsize_temp(t);
                end
            end
        end
    end
    clear tempvar_writeindex tempvar_grainsizes tempvar_lineids;
    
    linesampled_locs=margin:spacing_45:d2-margin-1;
    tempvar_writeindex={};
    tempvar_grainsizes={};
    tempvar_lineids={};
    %Measurement below diagonal
    parfor i=margin:d2-margin-1 %Start in bottom left corner, increase x
        %Crop xy data and logical ID's such that the corner of the data matches
        %with the starting point (can use diagonal matrix)
        %Flip to take opposite diagonal
        line_ids=diag(flip(ebsd_ids(margin:d1-margin,i:d2-margin)));
        point_x_line=diag(flip(x_grid(margin:d1-margin,i:d2-margin)));
        point_y_line=diag(flip(y_grid(margin:d1-margin,i:d2-margin)));
        xy1 = [point_x_line(1),point_y_line(1)];
        xy2 = [point_x_line(end),point_y_line(end)];
        
        %Measure intercepts for grain boundary
        [x_gb,y_gb,~] = intersect(gb,xy1,xy2);
        x_gb=x_gb(~isnan(x_gb));
        y_gb=y_gb(~isnan(y_gb));
        
        if include_innerGB == true
            %Measure intercepts for inner grain boundary
            [x_igb,y_igb,~] = intersect(gb_inner,xy1,xy2);
            x_igb=x_igb(~isnan(x_igb));
            y_igb=y_igb(~isnan(y_igb));
            
            if isempty(x_igb) == 0
                %Combine the measurements
                x_gb=horzcat(x_gb,x_igb);
                y_gb=horzcat(y_gb,y_igb);
                [x_gb,sort_index]=sort(x_gb);
                y_gb=y_gb(sort_index);
            else
                [x_gb,sort_index]=sort(x_gb);
                y_gb=y_gb(sort_index);
            end
        else
            [x_gb,sort_index]=sort(x_gb);
            y_gb=y_gb(sort_index);
        end
        
        %Find unique values (some problem in MTEX with duplicates)
        [x_gb_unique,ia,ic] = unique(x_gb);
        y_gb_unique=y_gb(ia);
        
        %Additional removing (somehow unique doesnt remove everything)
        [~,id_rem]=find(abs(diff(y_gb_unique))<0.1*stepsize_dy); %Delete points very close together
        x_gb_unique(id_rem)=[];
        y_gb_unique(id_rem)=[];
        
        %If intersect touches image margin, remove
        [~,rem_x_index]=find(x_gb_unique==point_x_line(1) | x_gb_unique==point_x_line(end));
        x_gb_unique(rem_x_index)=[];
        y_gb_unique(rem_x_index)=[];
        [~,rem_y_index]=find(y_gb_unique==point_y_line(1) | y_gb_unique==point_y_line(end));
        x_gb_unique(rem_y_index)=[];
        y_gb_unique(rem_y_index)=[];
        
        %Find the closest datapoint based on x y coordinates
        x_gb_unique=x_gb_unique+stepsize_dx/2; %Add half of stepsize to shift measuremens onto actual datapoints (for plotting purposes, matches direction of measurement).
        y_gb_unique=y_gb_unique-stepsize_dy/2; %Substract half of stepsize to shift measuremens onto actual datapoints (for plotting purposes, matches direction of measurement).
        [minValue, closestIndex] = min(abs(x_gb_unique - transpose(point_x_line).')); %Find closest indices, only need to do one direction
        
        %Calculate grain size
        hor_dist_45=diff(x_gb_unique);
        ver_dist_45=diff(y_gb_unique);
        grainsize_45_2_temp = sqrt((hor_dist_45.^2)+(ver_dist_45.^2));
        
        %Save values to temporary variables
        if isempty(closestIndex) == 0
            tempvar_writeindex{i}=closestIndex;
            tempvar_grainsizes{i}=grainsize_45_2_temp;
            tempvar_lineids{i}=line_ids;
        end
        
        %Only write the grain size values that are consistent with 7:10 line-spacing
        [~,valfound]=find(linesampled_locs==i);
        if isempty(grainsize_45_2_temp) == 0 && isempty(valfound) ==0
            grainsize_45_2=horzcat(grainsize_45_2,grainsize_45_2_temp);
        end
    end
    
    %Write values to GS map
    for i=1:length(tempvar_writeindex)
        closestIndex=tempvar_writeindex{i};
        grainsize_temp=tempvar_grainsizes{i};
        line_ids=tempvar_lineids{i};
        if isempty(closestIndex) == 0
            for t=1:length(grainsize_temp)
                write_index=closestIndex(t):closestIndex(t+1); %Output index
                if length(write_index)>min_measurement %At least n pixels required between the measurements
                    write_index(end)=[]; %Remove last index, so last measurement ends on the grain interior side
                    gsmap_45_2(line_ids(write_index))=grainsize_temp(t);
                end
            end
        end
    end
    clear tempvar_writeindex tempvar_grainsizes tempvar_lineids;
    
    a = toc;
    fprintf('Elapsed time %.0f seconds.\n', a);
    
    %% Data processing
    for zy=1:1
        fprintf('Post-processing data. ');
        tic
        %Non-measured areas replaced with NaN for plotting and post-processing
        gsmap_hor(find(gsmap_hor==0))=NaN;
        gsmap_ver(find(gsmap_ver==0))=NaN;
        gsmap_45_1(find(gsmap_45_1==0))=NaN;
        gsmap_45_2(find(gsmap_45_2==0))=NaN;
        
        %Calculate mean map
        clear gsmap_mean gsmap_stack;
        gsmap_stack(:,:,1)=gsmap_hor;
        gsmap_stack(:,:,2)=gsmap_ver;
        gsmap_stack(:,:,3)=gsmap_45_1;
        gsmap_stack(:,:,4)=gsmap_45_2;
        gsmap_mean=mean(gsmap_stack,3,'omitnan');
        gsmap_min=min(gsmap_stack,[],3,'omitnan');
        gsmap_max=max(gsmap_stack,[],3,'omitnan');
        grainsize_mean_ps=reshape(gsmap_mean,[],1);
        clear gsmap_stack;
        
        %Remove measurements below n*step size for point-sampled data
        grainsize_hor_ps=gsmap_hor(gsmap_hor>=min_measurement*stepsize_dx);
        grainsize_ver_ps=gsmap_ver(gsmap_ver>=min_measurement*stepsize_dy);
        grainsize_45_1_ps=gsmap_45_1(gsmap_45_1>=min_measurement*sqrt(2)*mean([stepsize_dx stepsize_dy])); %Diagonal direction considered with sqrt(2)
        grainsize_45_2_ps=gsmap_45_2(gsmap_45_2>=min_measurement*sqrt(2)*mean([stepsize_dx stepsize_dy])); %Diagonal direction considered with sqrt(2)
        
        %Combine all measurement directions into a single distribution
        grainsize_all_dirs_ps=horzcat(transpose(grainsize_hor_ps),transpose(grainsize_ver_ps));
        grainsize_all_dirs_ps=horzcat(grainsize_all_dirs_ps,transpose(grainsize_45_1_ps));
        grainsize_all_dirs_ps=horzcat(grainsize_all_dirs_ps,transpose(grainsize_45_2_ps));
        
        %Plotting limits for figures
        plot_lb=quantile(grainsize_all_dirs_ps.^-0.5,0.01);
        plot_ub=quantile(grainsize_all_dirs_ps.^-0.5,0.95);
        
        %--------------------------------
        %Remove values smaller than 3* step size for line-sampled data;
        grainsize_hor=grainsize_hor(grainsize_hor>=min_measurement*stepsize_dx);
        grainsize_ver=grainsize_ver(grainsize_ver>=min_measurement*stepsize_dy);
        grainsize_45_1=grainsize_45_1(grainsize_45_1>=min_measurement*sqrt(2)*mean([stepsize_dx stepsize_dy])); %Diagonal direction considered with sqrt(2)
        grainsize_45_2=grainsize_45_2(grainsize_45_2>=min_measurement*sqrt(2)*mean([stepsize_dx stepsize_dy])); %Diagonal direction considered with sqrt(2)
        
        grainsize = [grainsize_hor grainsize_ver grainsize_45_1 grainsize_45_2];
        
        average_grainsize = mean(grainsize); %Average grainsize output value
        volume_weighted_average = mean(grainsize_all_dirs_ps); %Volume-weighted average
        
        %Find different probability level grainsizes between 1-99.9%,
        grainsize_probability_level = []; %Output: Column 1: Probability level, Column 2: Grain size (m), Last row (index 101,2) = average grain size
        for value_to_find=0.01:0.01:0.99
            grainsize_probability_level(end+1,1) = value_to_find;
            grainsize_probability_level(end,2) = quantile(grainsize,value_to_find);
        end
        grainsize_probability_level(100,1)= 0.999;
        grainsize_probability_level(100,2)= quantile(grainsize,0.999);
        grainsize_probability_level(101,2)= average_grainsize;
        Dd_d = (grainsize_probability_level(99,2)-grainsize_probability_level(1,2))/average_grainsize;
        
        a = toc;
        fprintf('Elapsed time %.0f seconds.\n', a);
    end
end

%% Show figures and moving averages
for zz=1:1
    if figure_show==true
        %% Show grain size maps
        for xx=1:1
            fprintf('Showing figures. ');
            tic
            figure;
            plot(ebsd_grid,gsmap_hor.^-0.5);
            title('Horizontal grain size (m^{-0.5})');
            hold on;
            plot(grains_grid(grain_ids).boundary,'k','linewidth',gb_width);
            if include_innerGB == true
                plot(grains_grid(grain_ids).innerBoundary,'k','linewidth',gb_width);
            end
            mtexColorMap(flipud(jet));
            CLim(gcm,[plot_lb,plot_ub]);
            
            nextAxis;
            plot(ebsd_grid,gsmap_ver.^-0.5);
            title('Vertical grain size (m^{-0.5})');
            hold on;
            plot(grains_grid(grain_ids).boundary,'k','linewidth',gb_width);
            if include_innerGB == true
                plot(grains_grid(grain_ids).innerBoundary,'k','linewidth',gb_width);
            end
            mtexColorMap(flipud(jet));
            CLim(gcm,[plot_lb,plot_ub]);
            
            nextAxis;
            plot(ebsd_grid,gsmap_45_1.^-0.5);
            title('45 degree #1 grain size (m^{-0.5})');
            hold on;
            plot(grains_grid(grain_ids).boundary,'k','linewidth',gb_width);
            if include_innerGB == true
                plot(grains_grid(grain_ids).innerBoundary,'k','linewidth',gb_width);
            end
            mtexColorMap(flipud(jet));
            CLim(gcm,[plot_lb,plot_ub]);
            
            nextAxis;
            plot(ebsd_grid,gsmap_45_2.^-0.5);
            title('45 degree #2 grain size (m^{-0.5})');
            hold on;
            plot(grains_grid(grain_ids).boundary,'k','linewidth',gb_width);
            if include_innerGB == true
                plot(grains_grid(grain_ids).innerBoundary,'k','linewidth',gb_width);
            end
            mtexColorMap(flipud(jet));
            CLim(gcm,[plot_lb,plot_ub]);
            
            nextAxis;
            plot(ebsd_grid,gsmap_min.^-0.5);
            title('Minimum grain size (m^{-0.5})');
            hold on;
            plot(grains_grid(grain_ids).boundary,'k','linewidth',gb_width);
            if include_innerGB == true
                plot(grains_grid(grain_ids).innerBoundary,'k','linewidth',gb_width);
            end
            mtexColorMap(flipud(jet));
            CLim(gcm,[plot_lb,plot_ub]);
            
            nextAxis;
            plot(ebsd_grid,gsmap_max.^-0.5);
            title('Maximum grain size (m^{-0.5})');
            hold on;
            plot(grains_grid(grain_ids).boundary,'k','linewidth',gb_width);
            if include_innerGB == true
                plot(grains_grid(grain_ids).innerBoundary,'k','linewidth',gb_width);
            end
            mtexColorbar;
            mtexColorMap(flipud(jet));
            CLim(gcm,[plot_lb,plot_ub]);
            
            figure;
            plot(ebsd_grid,gsmap_mean.^-0.5);
            title('Mean grain size (m^{-0.5})');
            hold on;
            plot(grains_grid(grain_ids).boundary,'k','linewidth',gb_width);
            if include_innerGB == true
                plot(grains_grid(grain_ids).innerBoundary,'k','linewidth',gb_width);
            end
            mtexColorbar;
            mtexColorMap(flipud(jet));
            CLim(gcm,[plot_lb,plot_ub]);
            
            a = toc;
            fprintf('Elapsed time %.0f seconds.\n', a);
        end
        
        %% Show traces of moving averages
        % Colored line traces are shown to match direction of trace with the grain size maps
        % Only the for loop below needs to be executed if averaging_length is changed
        for xx=1:1
            %Moving averages taken further away from image margins in order to eliminate grains touching the edges.
            margin_2 = margin+ceil(mean(grainsize_all_dirs_ps)/stepsize_dx); %Volume-weighted average grain size is used as an added margin.
            averaging_length_1 = averaging_length -1;
            
            fprintf(['Calculating grain size moving averages, averaging length: ' num2str(averaging_length) ' pixels, x/y: ' num2str(averaging_length_1*ebsd_grid.dx) '/' num2str(averaging_length_1*ebsd_grid.dy) ' m.\n']);
            tic
            %% Calculate moving averages
            mov_avg_min_x_avg = {};
            mov_avg_mean_x_avg = {};
            mov_avg_max_x_avg = {};
            mov_avg_min_y_avg = {};
            mov_avg_mean_y_avg = {};
            mov_avg_max_y_avg = {};
            x_grid=ebsd_grid.x;
            y_grid=ebsd_grid.y;
            
            %Calculate vertical gradient
            for i = margin_2+1:1:d1-averaging_length_1-margin_2
                temp_block=gsmap_min(i:i+averaging_length_1,margin_2+1:d2-margin_2);
                temp_block_values = reshape(temp_block.',[],1);
                mov_avg_min_y_avg{1,end+1} = nanmean(temp_block_values)^-0.5;
                mov_avg_min_y_avg{2,end} = nanmean(mean(y_grid(i:i+averaging_length_1,margin_2+1:d2-margin_2)));
                
                temp_block=gsmap_mean(i:i+averaging_length_1,margin_2+1:d2-margin_2);
                temp_block_values = reshape(temp_block.',[],1);
                mov_avg_mean_y_avg{1,end+1} = nanmean(temp_block_values)^-0.5;
                mov_avg_mean_y_avg{2,end} = nanmean(mean(y_grid(i:i+averaging_length_1,margin_2+1:d2-margin_2)));
                
                temp_block=gsmap_max(i:i+averaging_length_1,margin_2+1:d2-margin_2);
                temp_block_values = reshape(temp_block.',[],1);
                mov_avg_max_y_avg{1,end+1} = nanmean(temp_block_values)^-0.5;
                mov_avg_max_y_avg{2,end} = nanmean(mean(y_grid(i:i+averaging_length_1,margin_2+1:d2-margin_2)));
            end
            
            %Calculate horizontal gradient
            for i = margin_2+1:1:d2-averaging_length_1-margin_2
                temp_block=gsmap_min(margin_2+1:d1-margin_2,i:i+averaging_length_1);
                temp_block_values = reshape(temp_block.',[],1);
                mov_avg_min_x_avg{1,end+1} = nanmean(temp_block_values)^-0.5;
                mov_avg_min_x_avg{2,end} = nanmean(nanmean(x_grid(margin_2+1:d1-margin_2,i:i+averaging_length_1)));
                
                temp_block=gsmap_mean(margin_2+1:d1-margin_2,i:i+averaging_length_1);
                temp_block_values = reshape(temp_block.',[],1);
                mov_avg_mean_x_avg{1,end+1} = nanmean(temp_block_values)^-0.5;
                mov_avg_mean_x_avg{2,end} = nanmean(nanmean(x_grid(margin_2+1:d1-margin_2,i:i+averaging_length_1)));
                
                temp_block=gsmap_max(margin_2+1:d1-margin_2,i:i+averaging_length_1);
                temp_block_values = reshape(temp_block.',[],1);
                mov_avg_max_x_avg{1,end+1} = nanmean(temp_block_values)^-0.5;
                mov_avg_max_x_avg{2,end} = nanmean(nanmean(x_grid(margin_2+1:d1-margin_2,i:i+averaging_length_1)));
            end
            
            %Convert cells to arrays
            mov_avg_min_x_avg = cell2mat(mov_avg_min_x_avg);
            mov_avg_mean_x_avg = cell2mat(mov_avg_mean_x_avg);
            mov_avg_max_x_avg = cell2mat(mov_avg_max_x_avg);
            mov_avg_min_y_avg = cell2mat(mov_avg_min_y_avg);
            mov_avg_mean_y_avg = cell2mat(mov_avg_mean_y_avg);
            mov_avg_max_y_avg = cell2mat(mov_avg_max_y_avg);
            
            %% Plot grain boundary map for vertical direction
            figure;
            plot(grains_grid(grain_ids).boundary,'k','linewidth',gb_width);
            hold on;
            title('Trace location for vertical grain size');
            if include_innerGB == true
                plot(grains_grid(grain_ids).innerBoundary,'k','linewidth',gb_width);
            end
            mtexColorbar;
            mtexColorbar;
            
            %Add line trace to visualize size and location of moving average window
            for tt=-2:2
                xy1=[ebsd_grid(margin_2,ceil(d2/2)+tt).x mov_avg_mean_y_avg(2,1)];
                xy2=[ebsd_grid(margin_2,ceil(d2/2)+tt).x mov_avg_mean_y_avg(2,end)];
                lineSec=[xy1;xy2];
                [ebsd_line,plot_coordinates] = spatialProfile(ebsd_grid('indexed'),lineSec);
                ebsd_line_ids=ebsd_line.id;
                temp_axis_colormap=jet(length(ebsd_line_ids));
                plot(ebsd_grid(ebsd_line_ids),temp_axis_colormap);
            end
            
            %Plot averaging window
            region = [mean(x_grid(:,margin_2+1)) mean(y_grid(margin_2+1,:)) (mean(x_grid(:,d2-margin_2))-mean(x_grid(:,margin_2+1)))  averaging_length_1*ebsd_grid.dy]; %First define a region by [xmin ymin xmax-xmin ymax-ymin]
            rectangle('position',region,'edgecolor','r','linewidth',2);
            
            %% Vertical grain size variation
            figure('color','w');
            plot([nanmean(grainsize_all_dirs_ps)^-0.5 nanmean(grainsize_all_dirs_ps)^-0.5],[min(mov_avg_mean_y_avg(2,:)) max(mov_avg_mean_y_avg(2,:))],'k:','LineWidth',1);
            hold on;
            title(['Vertical grain size, averaging length: ' num2str(averaging_length) ' pixels.']);
            plot(mov_avg_min_y_avg(1,:),mov_avg_min_y_avg(2,:),'r--','LineWidth',1);
            plot(mov_avg_mean_y_avg(1,:),mov_avg_mean_y_avg(2,:),'k','LineWidth',2);
            plot(mov_avg_max_y_avg(1,:),mov_avg_max_y_avg(2,:),'b--','LineWidth',1);
            min_val=0.9*min([mov_avg_max_x_avg(1,:) mov_avg_max_y_avg(1,:)]);
            max_val=1.1*max([mov_avg_min_x_avg(1,:) mov_avg_min_y_avg(1,:)]);
            
            %Plot line trace
            clear x y z;
            y = mov_avg_mean_y_avg(2,:);
            x = repelem(min_val,length(y));
            z = zeros(size(y));
            col = 1:length(y);  % This is the color, vary with x in this case.
            surface([x;x],[y;y],[z;z],[col;col],...
                'facecol','no',...
                'edgecol','interp',...
                'linew',4);
            colormap(jet);
            
            %Image formatting
            box on;
            hold off;
            ylabel('Y-coordinate (m)','FontSize',14,'FontWeight','bold');
            xlabel('\bfGrain size, \it d ^{-0.5} \rm \bf(m^{-0.5})','FontSize',14,'FontWeight','bold');
            set(gca,'Ydir','reverse');
            ylim([min(min(y_grid)) max(max(y_grid))]);
            xlim([min_val max_val]);
            legend('d_v','Minimum', 'Mean', 'Maximum','FontSize',10);
            
            %% Plot grain boundary map for horizontal direction
            figure;
            plot(grains_grid(grain_ids).boundary,'k','linewidth',gb_width);
            title('Trace location for horizontal grain size');
            hold on;
            if include_innerGB == true
                plot(grains_grid(grain_ids).innerBoundary,'k','linewidth',gb_width);
            end
            mtexColorbar;
            mtexColorbar;
            
            %Add line trace to visualize size and location of moving average window
            for tt=-2:2
                xy1=[mov_avg_min_x_avg(2,1) ebsd_grid(ceil(d1/2)+tt,margin_2).y]; %fix second coordinate
                xy2=[mov_avg_min_x_avg(2,end) ebsd_grid(ceil(d1/2)+tt,margin_2).y]; %fix second coordinate
                lineSec=[xy1;xy2];
                [ebsd_line,plot_coordinates] = spatialProfile(ebsd_grid('indexed'),lineSec);
                ebsd_line_ids=ebsd_line.id;
                temp_axis_colormap=parula(length(ebsd_line_ids)); %Parula used to distinct from vertical direction
                plot(ebsd_grid(ebsd_line_ids),temp_axis_colormap);
            end
            
            %Plot averaging window
            region = [mean(x_grid(:,margin_2+1)) mean(y_grid(margin_2+1,:)) averaging_length_1*ebsd_grid.dx (mean(y_grid(d1-margin_2,:))-mean(y_grid(margin_2+1,:)))]; %First define a region by [xmin ymin xmax-xmin ymax-ymin]
            rectangle('position',region,'edgecolor','r','linewidth',2);
            
            %% Horizontal grain size variation
            figure('color','w');
            plot([min(mov_avg_mean_x_avg(2,:)) max(mov_avg_mean_x_avg(2,:))], [nanmean(grainsize_all_dirs_ps)^-0.5 nanmean(grainsize_all_dirs_ps)^-0.5],'k:','LineWidth',1);
            hold on;
            title(['Horizontal grain size, averaging length: ' num2str(averaging_length) ' pixels.']);
            plot(mov_avg_min_x_avg(2,:),mov_avg_min_x_avg(1,:),'r--','LineWidth',1);
            plot(mov_avg_mean_x_avg(2,:),mov_avg_mean_x_avg(1,:),'k','LineWidth',2);
            plot(mov_avg_max_x_avg(2,:),mov_avg_max_x_avg(1,:),'b--','LineWidth',1);
            
            %Plot line trace
            clear x y z;
            x = mov_avg_mean_x_avg(2,:);
            y = repelem(min_val,length(x));
            z = zeros(size(x));
            col = 1:length(y);  % This is the color, vary with x in this case.
            surface([x;x],[y;y],[z;z],[col;col],...
                'facecol','no',...
                'edgecol','interp',...
                'linew',4);
            colormap(parula);
            
            %Image formatting
            box on;
            hold off;
            ylabel('\bfGrain size, \it d ^{-0.5} \rm \bf(m^{-0.5})','FontSize',14,'FontWeight','bold');
            xlabel('X-coordinate (m)','FontSize',14,'FontWeight','bold');
            xlim([min(min(x_grid)) max(max(x_grid))]);
            ylim([min_val max_val]);
            legend('d_v','Minimum', 'Mean', 'Maximum','FontSize',10);
        end
    end
end

%% Print output and clear variables
fprintf('The average grain size is %.2f m\n',average_grainsize);
fprintf('The relative grain size dispersion is %.2f\n',Dd_d);
fprintf('The volume-weighted average grain size is %.2f m.\n',volume_weighted_average);

